@@ -1,5 +1,5 @@
 #
-# Copyright (c), 2016-2020, SISSA (International School for Advanced Studies).
+# Copyright (c), 2016-2026, SISSA (International School for Advanced Studies).
 # All rights reserved.
 # This file is distributed under the terms of the MIT License.
 # See the file 'LICENSE' in the root directory of the present
@@ -17,7 +17,7 @@ from collections import Counter
 from collections.abc import Iterator
 from typing import TYPE_CHECKING, cast, Any, Optional, Union
 
-from elementpath import ElementPathError, XPathContext, XPathToken, \
+from elementpath import ElementPathError, XPathContext, \
     ElementNode, translate_pattern, AttributeNode
 from elementpath.datatypes import UntypedAtomic
 from elementpath.xpath_nodes import EtreeElementNode
@@ -48,20 +48,6 @@ IdentityCounterType = tuple[IdentityFieldItemType, ...]
 # not used for the specific selection of fields and elements and the XSD
 # fields are collected at first validation run.
 
-def iter_root_elements(token: XPathToken) -> Iterator[XPathToken]:
-    if token.symbol in ('(name)', ':', '*', '.'):
-        yield token
-    elif token.symbol in ('//', '/'):
-        yield from iter_root_elements(token[0])
-        for tk in token[1].iter():
-            if tk.symbol == '|':
-                yield from iter_root_elements(tk[1])
-                break
-    elif token.symbol in '|':
-        for tk in token:
-            yield from iter_root_elements(tk)
-
-
 IdentityMapType = dict[Union['XsdKey', 'XsdKeyref', str, None],
                        Union['IdentityCounter', 'KeyrefCounter']]
 IdentityNodeType = Union[ElementNode, AttributeNode]
@@ -190,32 +176,39 @@ class XsdIdentity(XsdComponent):
         self.elements = {}
 
     def build(self) -> None:
-        if self._built:
+        if self._built is not False:
             return
-        self._built = True
-
-        if self.ref is True:  # type: ignore[comparison-overlap]
-            try:
-                ref = self.maps.identities[self.name]
-            except KeyError:
-                self.fields = []
-                self.elements = {}
-                msg = _("unknown identity constraint {!r}")
-                self.parse_error(msg.format(self.name))
-                return
-            else:
-                if not isinstance(ref, self.__class__):
-                    msg = _("attribute 'ref' points to a different kind constraint")
-                    self.parse_error(msg)
-                self.selector = ref.selector
-                self.fields = ref.fields
-                self.elements = {}
-                self.ref = ref
+        self._built = None
 
         try:
-            self.update_elements(base_element=self.parent)
-        except TypeError as err:
-            self.parse_error(err)
+            if self.ref is self:
+                try:
+                    ref = self.maps.identities[self.name]
+                except KeyError:
+                    self.fields = []
+                    self.elements = {}
+                    msg = _("unknown identity constraint {!r}")
+                    self.parse_error(msg.format(self.name))
+                    self.ref = None
+                    return
+                else:
+                    if not isinstance(ref, self.__class__):
+                        msg = _("attribute 'ref' points to a different kind constraint")
+                        self.parse_error(msg)
+                    self.selector = ref.selector
+                    self.fields = ref.fields
+                    self.elements = {}
+                    self.ref = ref
+
+            try:
+                self.update_elements(base_element=self.parent)
+            except TypeError as err:
+                self.parse_error(err)
+
+            self._built = True
+        finally:
+            if self._built is None:
+                self._built = False
 
     def update_elements(self, base_element: Union['XsdElement', XPathElement]) -> None:
         if self.selector is None:
@@ -272,7 +265,7 @@ class XsdKeyref(XsdIdentity):
     or in a descendant element.
     """
     _ADMITTED_TAGS = nm.XSD_KEYREF,
-    refer: Optional[Union[str, XsdKey]] = None
+    refer: str | XsdKey | None = None
     refer_path = '.'
 
     def _parse(self) -> None:
@@ -347,7 +340,7 @@ class XsdKeyref(XsdIdentity):
 class Xsd11Unique(XsdUnique):
     def _parse(self) -> None:
         if self._parse_reference():
-            self.ref = True  # type: ignore[assignment]
+            self.ref = self
         else:
             super()._parse()
 
@@ -355,7 +348,7 @@ class Xsd11Unique(XsdUnique):
 class Xsd11Key(XsdKey):
     def _parse(self) -> None:
         if self._parse_reference():
-            self.ref = True  # type: ignore[assignment]
+            self.ref = self
         else:
             super()._parse()
 
@@ -363,7 +356,7 @@ class Xsd11Key(XsdKey):
 class Xsd11Keyref(XsdKeyref):
     def _parse(self) -> None:
         if self._parse_reference():
-            self.ref = True  # type: ignore[assignment]
+            self.ref = self
         else:
             super()._parse()
 
@@ -526,24 +519,26 @@ class FieldValueSelector:
             if empty:
                 value = self.value_constraints.get(None)
 
-        if value is None:
-            if not isinstance(self.field.parent, XsdKey) or \
-                    'ref' in element_node.obj.attrib and \
-                    self.field.schema.meta_schema is None and \
-                    self.field.schema.XSD_VERSION != '1.0':
-                return None
-            else:
-                msg = _("missing key field {0!r} for {1!r}")
-                raise XMLSchemaValueError(msg.format(self.field.path, self))
-        elif isinstance(value, list):
-            return tuple(value)
-        elif isinstance(value, UntypedAtomic):
-            return str(value)
-        elif isinstance(value, bool):
-            return value, bool
-        elif not isinstance(value, float):
-            return value
-        elif math.isnan(value):
-            return 'nan', float
-        else:
-            return value, float
+        match value:
+            case None:
+                if not isinstance(self.field.parent, XsdKey) or \
+                        'ref' in element_node.obj.attrib and \
+                        self.field.schema.meta_schema is None and \
+                        self.field.schema.XSD_VERSION != '1.0':
+                    return None
+                else:
+                    msg = _("missing key field {0!r} for {1!r}")
+                    raise XMLSchemaValueError(msg.format(self.field.path, self))
+            case list():
+                return tuple(value)
+            case UntypedAtomic():
+                return str(value)
+            case bool():
+                return value, bool
+            case float():
+                if math.isnan(value):
+                    return 'nan', float
+                else:
+                    return value, float
+            case _:
+                return value
