@@ -1,5 +1,5 @@
 #
-# Copyright (c), 2016-2020, SISSA (International School for Advanced Studies).
+# Copyright (c), 2016-2026, SISSA (International School for Advanced Studies).
 # All rights reserved.
 # This file is distributed under the terms of the MIT License.
 # See the file 'LICENSE' in the root directory of the present
@@ -21,6 +21,7 @@ from xmlschema.translation import gettext as _
 from xmlschema.utils.qnames import get_namespace
 from xmlschema.utils.decoding import EmptyType, Empty, raw_encode_value
 from xmlschema.xpath import XMLSchemaProxy, ElementPathMixin
+from xmlschema.caching import schema_cache
 
 from .validation import ValidationContext, EncodeContext, ValidationMixin
 from .xsdbase import XsdComponent
@@ -69,28 +70,27 @@ class XsdWildcard(XsdComponent):
 
     def _parse(self) -> None:
         # Parse namespace and processContents
-        namespace = self.elem.attrib.get('namespace', '##any').strip()
-
-        if namespace in ('##any', '##other'):
-            self.namespace = {namespace}
-        elif not namespace:
-            self.namespace = set()  # an empty value means no namespace allowed!
-        elif namespace == '##local':
-            self.namespace = {''}
-        elif namespace == '##targetNamespace':
-            self.namespace = {self.target_namespace}
-        else:
-            self.namespace = set()
-            for ns in namespace.split():
-                if ns == '##local':
-                    self.namespace.add('')
-                elif ns == '##targetNamespace':
-                    self.namespace.add(self.target_namespace)
-                elif ns.startswith('##'):
-                    msg = _("wrong value %r in 'namespace' attribute")
-                    self.parse_error(msg % ns)
-                else:
-                    self.namespace.add(ns)
+        match namespace := self.elem.attrib.get('namespace', '##any').strip():
+            case '##any' | '##other':
+                self.namespace = {namespace}
+            case '':
+                self.namespace = set()  # an empty value means no namespace allowed!
+            case '##local':
+                self.namespace = {''}
+            case '##targetNamespace':
+                self.namespace = {self.target_namespace}
+            case _:
+                self.namespace = set()
+                for ns in namespace.split():
+                    if ns == '##local':
+                        self.namespace.add('')
+                    elif ns == '##targetNamespace':
+                        self.namespace.add(self.target_namespace)
+                    elif ns.startswith('##'):
+                        msg = _("wrong value %r in 'namespace' attribute")
+                        self.parse_error(msg % ns)
+                    else:
+                        self.namespace.add(ns)
 
         self.process_contents = self.elem.attrib.get('processContents', 'strict')
         if self.process_contents not in ('strict', 'lax', 'skip'):
@@ -211,6 +211,7 @@ class XsdWildcard(XsdComponent):
     def _has_occurs_restriction(self, other: 'XsdWildcard') -> bool:
         return True
 
+    @schema_cache
     def is_restriction(self, other: Union[ModelParticleType, 'XsdAnyAttribute'],
                        check_occurs: bool = True) -> bool:
         if not isinstance(other, self.__class__):
@@ -581,6 +582,7 @@ class XsdAnyElement(XsdWildcard, ParticleMixin,
 
         return self.maps.any_type.raw_encode(obj, validation, context)
 
+    @schema_cache
     def is_overlap(self, other: ModelParticleType) -> bool:
         if not isinstance(other, XsdAnyElement):
             if isinstance(other, elements.XsdElement):
@@ -884,6 +886,7 @@ class XsdOpenContent(XsdComponent):
         else:
             self.any_element = Xsd11AnyElement(child, self.schema, self)
 
+    @schema_cache
     def is_restriction(self, other: 'XsdOpenContent') -> bool:
         if other is None or other.mode == 'none':
             return self.mode == 'none'
